@@ -6,23 +6,24 @@
 // spell-checker:ignore (ToDOs) corasick memchr Roff trunc oset iset CHARCLASS
 
 use std::cmp;
+use std::cmp::PartialEq;
 use std::collections::{BTreeSet, HashMap, HashSet};
+use std::ffi::{OsStr, OsString};
 use std::fmt::Write as FmtWrite;
 use std::fs::File;
 use std::io::{BufRead, BufReader, BufWriter, Read, Write, stdin, stdout};
 use std::num::ParseIntError;
+use std::path::Path;
 
 use clap::{Arg, ArgAction, Command};
 use regex::Regex;
 use thiserror::Error;
 use uucore::display::Quotable;
 use uucore::error::{FromIo, UError, UResult, UUsageError};
-use uucore::{format_usage, help_about, help_usage};
+use uucore::format_usage;
+use uucore::translate;
 
-const USAGE: &str = help_usage!("ptx.md");
-const ABOUT: &str = help_about!("ptx.md");
-
-#[derive(Debug)]
+#[derive(Debug, PartialEq)]
 enum OutFormat {
     Dumb,
     Roff,
@@ -67,13 +68,12 @@ fn read_word_filter_file(
     option: &str,
 ) -> std::io::Result<HashSet<String>> {
     let filename = matches
-        .get_one::<String>(option)
-        .expect("parsing options failed!")
-        .to_string();
+        .get_one::<OsString>(option)
+        .expect("parsing options failed!");
     let reader: BufReader<Box<dyn Read>> = BufReader::new(if filename == "-" {
         Box::new(stdin())
     } else {
-        let file = File::open(filename)?;
+        let file = File::open(Path::new(filename))?;
         Box::new(file)
     });
     let mut words: HashSet<String> = HashSet::new();
@@ -89,12 +89,12 @@ fn read_char_filter_file(
     option: &str,
 ) -> std::io::Result<HashSet<char>> {
     let filename = matches
-        .get_one::<String>(option)
+        .get_one::<OsString>(option)
         .expect("parsing options failed!");
     let mut reader: Box<dyn Read> = if filename == "-" {
         Box::new(stdin())
     } else {
-        let file = File::open(filename)?;
+        let file = File::open(Path::new(filename))?;
         Box::new(file)
     };
     let mut buffer = String::new();
@@ -152,7 +152,7 @@ impl WordFilter {
                     if v.is_empty() {
                         None
                     } else {
-                        Some(v.to_string())
+                        Some(v.to_owned())
                     }
                 }
                 None => None,
@@ -192,15 +192,17 @@ struct WordRef {
     local_line_nr: usize,
     position: usize,
     position_end: usize,
-    filename: String,
+    filename: OsString,
 }
 
 #[derive(Debug, Error)]
 enum PtxError {
-    #[error("There is no dumb format with GNU extensions disabled")]
+    #[error("{}", translate!("ptx-error-dumb-format"))]
     DumbFormat,
-    #[error("{0} not implemented yet")]
+
+    #[error("{}", translate!("ptx-error-not-implemented", "feature" => (*.0)))]
     NotImplemented(&'static str),
+
     #[error("{0}")]
     ParseError(ParseIntError),
 }
@@ -222,19 +224,19 @@ fn get_config(matches: &clap::ArgMatches) -> UResult<Config> {
     }
     config.auto_ref = matches.get_flag(options::AUTO_REFERENCE);
     config.input_ref = matches.get_flag(options::REFERENCES);
-    config.right_ref &= matches.get_flag(options::RIGHT_SIDE_REFS);
+    config.right_ref = matches.get_flag(options::RIGHT_SIDE_REFS);
     config.ignore_case = matches.get_flag(options::IGNORE_CASE);
     if matches.contains_id(options::MACRO_NAME) {
-        config.macro_name = matches
+        matches
             .get_one::<String>(options::MACRO_NAME)
             .expect(err_msg)
-            .to_string();
+            .clone_into(&mut config.macro_name);
     }
     if matches.contains_id(options::FLAG_TRUNCATION) {
-        config.trunc_str = matches
+        matches
             .get_one::<String>(options::FLAG_TRUNCATION)
             .expect(err_msg)
-            .to_string();
+            .clone_into(&mut config.trunc_str);
     }
     if matches.contains_id(options::WIDTH) {
         config.line_width = matches
@@ -272,16 +274,16 @@ struct FileContent {
     offset: usize,
 }
 
-type FileMap = HashMap<String, FileContent>;
+type FileMap = HashMap<OsString, FileContent>;
 
-fn read_input(input_files: &[String]) -> std::io::Result<FileMap> {
+fn read_input(input_files: &[OsString]) -> std::io::Result<FileMap> {
     let mut file_map: FileMap = HashMap::new();
     let mut offset: usize = 0;
     for filename in input_files {
         let reader: BufReader<Box<dyn Read>> = BufReader::new(if filename == "-" {
             Box::new(stdin())
         } else {
-            let file = File::open(filename)?;
+            let file = File::open(Path::new(filename))?;
             Box::new(file)
         });
         let lines: Vec<String> = reader.lines().collect::<std::io::Result<Vec<String>>>()?;
@@ -291,7 +293,7 @@ fn read_input(input_files: &[String]) -> std::io::Result<FileMap> {
         let chars_lines: Vec<Vec<char>> = lines.iter().map(|x| x.chars().collect()).collect();
         let size = lines.len();
         file_map.insert(
-            filename.to_owned(),
+            filename.clone(),
             FileContent {
                 lines,
                 chars_lines,
@@ -350,11 +352,15 @@ fn create_word_set(config: &Config, filter: &WordFilter, file_map: &FileMap) ->
 
 fn get_reference(config: &Config, word_ref: &WordRef, line: &str, context_reg: &Regex) -> String {
     if config.auto_ref {
-        format!(
-            "{}:{}",
-            word_ref.filename.maybe_quote(),
-            word_ref.local_line_nr + 1
-        )
+        if word_ref.filename == "-" {
+            format!(":{}", word_ref.local_line_nr + 1)
+        } else {
+            format!(
+                "{}:{}",
+                word_ref.filename.maybe_quote(),
+                word_ref.local_line_nr + 1
+            )
+        }
     } else if config.input_ref {
         let (beg, end) = match context_reg.find(line) {
             Some(x) => (x.start(), x.end()),
@@ -402,7 +408,7 @@ fn trim_idx(s: &[char], beg: usize, end: usize) -> (usize, usize) {
     while b < e && s[b].is_whitespace() {
         b += 1;
     }
-    while b < e && s[e - 1].is_whitespace() {
+    while beg < e && s[e - 1].is_whitespace() {
         e -= 1;
     }
     (b, e)
@@ -482,7 +488,18 @@ fn get_output_chunks(
     let tail_end = trim_broken_word_right(all_after, tail_beg, tail_end);
 
     // trim away whitespace again.
-    let (tail_beg, tail_end) = trim_idx(all_after, tail_beg, tail_end);
+    let (tail_beg, mut tail_end) = trim_idx(all_after, tail_beg, tail_end);
+    // Fix: Manually trim trailing char (like "a") that are preceded by a space.
+    // This handles cases like "is a" which are not correctly trimmed by the
+    // preceding functions.
+    if tail_end >= 2
+        && (tail_end - 2) > tail_beg
+        && all_after[tail_end - 2].is_whitespace()
+        && !all_after[tail_end - 1].is_whitespace()
+    {
+        tail_end -= 1;
+        (_, tail_end) = trim_idx(all_after, tail_beg, tail_end);
+    }
 
     // and get the string
     let tail_str: String = all_after[tail_beg..tail_end].iter().collect();
@@ -510,19 +527,21 @@ fn get_output_chunks(
     // and get the string.
     let head_str: String = all_before[head_beg..head_end].iter().collect();
     head.push_str(&head_str);
+    //The TeX mode does not output truncation characters.
+    if config.format != OutFormat::Tex {
+        // put right context truncation string if needed
+        if after_end != all_after.len() && tail_beg == tail_end {
+            after.push_str(&config.trunc_str);
+        } else if after_end != all_after.len() && tail_end != all_after.len() {
+            tail.push_str(&config.trunc_str);
+        }
 
-    // put right context truncation string if needed
-    if after_end != all_after.len() && tail_beg == tail_end {
-        after.push_str(&config.trunc_str);
-    } else if after_end != all_after.len() && tail_end != all_after.len() {
-        tail.push_str(&config.trunc_str);
-    }
-
-    // put left context truncation string if needed
-    if before_beg != 0 && head_beg == head_end {
-        before = format!("{}{before}", config.trunc_str);
-    } else if before_beg != 0 && head_beg != 0 {
-        head = format!("{}{head}", config.trunc_str);
+        // put left context truncation string if needed
+        if before_beg != 0 && head_beg == head_end {
+            before = format!("{}{before}", config.trunc_str);
+        } else if before_beg != 0 && head_beg != 0 {
+            head = format!("{}{head}", config.trunc_str);
+        }
     }
 
     (tail, before, after, head)
@@ -642,24 +661,34 @@ fn prepare_line_chunks(
 }
 
 fn write_traditional_output(
-    config: &Config,
+    config: &mut Config,
     file_map: &FileMap,
     words: &BTreeSet<WordRef>,
-    output_filename: &str,
+    output_filename: &OsStr,
 ) -> UResult<()> {
-    let mut writer: BufWriter<Box<dyn Write>> = BufWriter::new(if output_filename == "-" {
-        Box::new(stdout())
-    } else {
-        let file = File::create(output_filename)
-            .map_err_context(|| output_filename.maybe_quote().to_string())?;
-        Box::new(file)
-    });
+    let mut writer: BufWriter<Box<dyn Write>> =
+        BufWriter::new(if output_filename == OsStr::new("-") {
+            Box::new(stdout())
+        } else {
+            let file = File::create(output_filename)
+                .map_err_context(|| output_filename.to_string_lossy().quote().to_string())?;
+            Box::new(file)
+        });
 
     let context_reg = Regex::new(&config.context_regex).unwrap();
 
+    if !config.right_ref {
+        let max_ref_len = if config.auto_ref {
+            get_auto_max_reference_len(words)
+        } else {
+            0
+        };
+        config.line_width -= max_ref_len;
+    }
+
     for word_ref in words {
         let file_map_value: &FileContent = file_map
-            .get(&(word_ref.filename))
+            .get(&word_ref.filename)
             .expect("Missing file in file map");
         let FileContent {
             ref lines,
@@ -691,14 +720,42 @@ fn write_traditional_output(
                 return Err(PtxError::DumbFormat.into());
             }
         };
-        writeln!(writer, "{output_line}").map_err_context(|| "write failed".into())?;
+        writeln!(writer, "{output_line}")
+            .map_err_context(|| translate!("ptx-error-write-failed"))?;
     }
 
-    writer.flush().map_err_context(|| "write failed".into())?;
+    writer
+        .flush()
+        .map_err_context(|| translate!("ptx-error-write-failed"))?;
 
     Ok(())
 }
 
+fn get_auto_max_reference_len(words: &BTreeSet<WordRef>) -> usize {
+    //Get the maximum length of the reference field
+    let line_num = words
+        .iter()
+        .map(|w| {
+            if w.local_line_nr == 0 {
+                1
+            } else {
+                (w.local_line_nr as f64).log10() as usize + 1
+            }
+        })
+        .max()
+        .unwrap_or(0);
+
+    let filename_len = words
+        .iter()
+        .filter(|w| w.filename != "-")
+        .map(|w| w.filename.maybe_quote().to_string().len())
+        .max()
+        .unwrap_or(0);
+
+    // +1 for the colon
+    line_num + filename_len + 1
+}
+
 mod options {
     pub mod format {
         pub static ROFF: &str = "roff";
@@ -725,82 +782,84 @@ mod options {
 
 #[uucore::main]
 pub fn uumain(args: impl uucore::Args) -> UResult<()> {
-    let matches = uu_app().try_get_matches_from(args)?;
-    let config = get_config(&matches)?;
+    let matches = uucore::clap_localization::handle_clap_result(uu_app(), args)?;
+    let mut config = get_config(&matches)?;
 
     let input_files;
-    let output_file;
+    let output_file: OsString;
 
     let mut files = matches
-        .get_many::<String>(options::FILE)
+        .get_many::<OsString>(options::FILE)
         .into_iter()
         .flatten()
         .cloned();
 
-    if !config.gnu_ext {
-        input_files = vec![files.next().unwrap_or("-".to_string())];
-        output_file = files.next().unwrap_or("-".to_string());
-        if let Some(file) = files.next() {
-            return Err(UUsageError::new(
-                1,
-                format!("extra operand {}", file.quote()),
-            ));
-        }
-    } else {
+    if config.gnu_ext {
         input_files = {
             let mut files = files.collect::<Vec<_>>();
             if files.is_empty() {
-                files.push("-".to_string());
+                files.push(OsString::from("-"));
             }
             files
         };
-        output_file = "-".to_string();
+        output_file = OsString::from("-");
+    } else {
+        input_files = vec![files.next().unwrap_or(OsString::from("-"))];
+        output_file = files.next().unwrap_or(OsString::from("-"));
+        if let Some(file) = files.next() {
+            return Err(UUsageError::new(
+                1,
+                translate!("ptx-error-extra-operand", "operand" => file.to_string_lossy().quote()),
+            ));
+        }
     }
 
     let word_filter = WordFilter::new(&matches, &config)?;
     let file_map = read_input(&input_files).map_err_context(String::new)?;
     let word_set = create_word_set(&config, &word_filter, &file_map);
-    write_traditional_output(&config, &file_map, &word_set, &output_file)
+    write_traditional_output(&mut config, &file_map, &word_set, &output_file)
 }
 
 pub fn uu_app() -> Command {
     Command::new(uucore::util_name())
-        .about(ABOUT)
+        .about(translate!("ptx-about"))
         .version(uucore::crate_version!())
-        .override_usage(format_usage(USAGE))
+        .help_template(uucore::localized_help_template(uucore::util_name()))
+        .override_usage(format_usage(&translate!("ptx-usage")))
         .infer_long_args(true)
         .arg(
             Arg::new(options::FILE)
                 .hide(true)
                 .action(ArgAction::Append)
-                .value_hint(clap::ValueHint::FilePath),
+                .value_hint(clap::ValueHint::FilePath)
+                .value_parser(clap::value_parser!(OsString)),
         )
         .arg(
             Arg::new(options::AUTO_REFERENCE)
                 .short('A')
                 .long(options::AUTO_REFERENCE)
-                .help("output automatically generated references")
+                .help(translate!("ptx-help-auto-reference"))
                 .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new(options::TRADITIONAL)
                 .short('G')
                 .long(options::TRADITIONAL)
-                .help("behave more like System V 'ptx'")
+                .help(translate!("ptx-help-traditional"))
                 .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new(options::FLAG_TRUNCATION)
                 .short('F')
                 .long(options::FLAG_TRUNCATION)
-                .help("use STRING for flagging line truncations")
+                .help(translate!("ptx-help-flag-truncation"))
                 .value_name("STRING"),
         )
         .arg(
             Arg::new(options::MACRO_NAME)
                 .short('M')
                 .long(options::MACRO_NAME)
-                .help("macro name to use instead of 'xx'")
+                .help(translate!("ptx-help-macro-name"))
                 .value_name("STRING"),
         )
         .arg(
@@ -813,14 +872,14 @@ pub fn uu_app() -> Command {
         .arg(
             Arg::new(options::format::ROFF)
                 .short('O')
-                .help("generate output as roff directives")
+                .help(translate!("ptx-help-roff"))
                 .overrides_with_all([options::FORMAT, options::format::ROFF, options::format::TEX])
                 .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new(options::format::TEX)
                 .short('T')
-                .help("generate output as TeX directives")
+                .help(translate!("ptx-help-tex"))
                 .overrides_with_all([options::FORMAT, options::format::ROFF, options::format::TEX])
                 .action(ArgAction::SetTrue),
         )
@@ -828,66 +887,69 @@ pub fn uu_app() -> Command {
             Arg::new(options::RIGHT_SIDE_REFS)
                 .short('R')
                 .long(options::RIGHT_SIDE_REFS)
-                .help("put references at right, not counted in -w")
+                .help(translate!("ptx-help-right-side-refs"))
                 .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new(options::SENTENCE_REGEXP)
                 .short('S')
                 .long(options::SENTENCE_REGEXP)
-                .help("for end of lines or end of sentences")
+                .help(translate!("ptx-help-sentence-regexp"))
                 .value_name("REGEXP"),
         )
         .arg(
             Arg::new(options::WORD_REGEXP)
                 .short('W')
                 .long(options::WORD_REGEXP)
-                .help("use REGEXP to match each keyword")
+                .help(translate!("ptx-help-word-regexp"))
                 .value_name("REGEXP"),
         )
         .arg(
             Arg::new(options::BREAK_FILE)
                 .short('b')
                 .long(options::BREAK_FILE)
-                .help("word break characters in this FILE")
+                .help(translate!("ptx-help-break-file"))
                 .value_name("FILE")
-                .value_hint(clap::ValueHint::FilePath),
+                .value_hint(clap::ValueHint::FilePath)
+                .value_parser(clap::value_parser!(OsString)),
         )
         .arg(
             Arg::new(options::IGNORE_CASE)
                 .short('f')
                 .long(options::IGNORE_CASE)
-                .help("fold lower case to upper case for sorting")
+                .help(translate!("ptx-help-ignore-case"))
                 .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new(options::GAP_SIZE)
                 .short('g')
                 .long(options::GAP_SIZE)
-                .help("gap size in columns between output fields")
+                .help(translate!("ptx-help-gap-size"))
                 .value_name("NUMBER"),
         )
         .arg(
             Arg::new(options::IGNORE_FILE)
                 .short('i')
                 .long(options::IGNORE_FILE)
-                .help("read ignore word list from FILE")
+                .help(translate!("ptx-help-ignore-file"))
                 .value_name("FILE")
-                .value_hint(clap::ValueHint::FilePath),
+                .value_hint(clap::ValueHint::FilePath)
+                .value_parser(clap::value_parser!(OsString)),
         )
         .arg(
             Arg::new(options::ONLY_FILE)
                 .short('o')
                 .long(options::ONLY_FILE)
-                .help("read only word list from this FILE")
+                .help(translate!("ptx-help-only-file"))
                 .value_name("FILE")
-                .value_hint(clap::ValueHint::FilePath),
+                .value_hint(clap::ValueHint::FilePath)
+                .value_parser(clap::value_parser!(OsString)),
         )
         .arg(
             Arg::new(options::REFERENCES)
                 .short('r')
                 .long(options::REFERENCES)
-                .help("first field of each line is a reference")
+                .help(translate!("ptx-help-references"))
                 .value_name("FILE")
                 .action(ArgAction::SetTrue),
         )
@@ -895,7 +957,7 @@ pub fn uu_app() -> Command {
             Arg::new(options::WIDTH)
                 .short('w')
                 .long(options::WIDTH)
-                .help("output width in columns, reference excluded")
+                .help(translate!("ptx-help-width"))
                 .value_name("NUMBER"),
         )
 }
